// 
// Copyright (C) 2013 UMPC
// 
// This file is part of CLaRiNet.
//  
// CLaRiNet is free software: you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// CLaRiNet is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with CLaRiNet.  If not, see <http://www.gnu.org/licenses/>.
function LocalP = BuildLocalRainfall ( PRECIP , XYZ_precip , XYZ_estim , ParamPrecip , MeanVariancePrecip )
    
    // PRECIP            :     Matrix [ndays,nprecip]    :    Daily rainfall time series at gauges
    // XYZ_precip        :     Matrix [nprecip,3]        :    X,Y,Z coordinates of rain gauges
    // XYZ_estim         :     Matrix [nestim,3]         :    XYZ of ungaged locations
    //
    // ParamPrecip       :     Vector [3]                :    ParamPrecip(1)    =>    Rainfall characteristic height
    //                                                        ParamPrecip(2)    =>    Variogram range for precipitation template residuals
    //                                                        ParamPrecip(3)    =>    Variogram range for anamorphosed local scaling factor for WP i
    //
    // MeanVariancePrecip :    Vector [4]                :    Parameters of the mean-variance relationship of the local scaling factor
    
    PRECIP(PRECIP<0) = %nan ;
    
    if or(isnan(PRECIP)) then
        error("Les series pluviometriques ne doivent pas comporter de lacunes");
    end
    
    [ndays,nprecip] = size(PRECIP);
    nestim = size(XYZ_estim,'r');
    
    if or( size(XYZ_precip) ~= [nprecip,3] ) then
        error("Les arguments 1 et 3 ont des dimensions incompatibles");
    end

    //////////////////////////////////////////////////////////////////////////// 
    ///////////// Compute sample-sample symmetric distance matrices ////////////
    ////////////////////////////////////////////////////////////////////////////
    
    DX = XYZ_precip(:,1)'.*.ones(nprecip,1)-XYZ_precip(:,1).*.ones(1,nprecip);
    DY = XYZ_precip(:,2)'.*.ones(nprecip,1)-XYZ_precip(:,2).*.ones(1,nprecip);
    D_pp = sqrt(DX.^2+DY.^2);
            
    // Dimensioning output:
    
    LocalP = zeros(ndays,nestim);
    
    //////////////////////////////////////////////////////////////////////////// 
    /// Computing Gauss-Hermite quadrature abcissas and weights once for all ///
    ////////////////////////////////////////////////////////////////////////////
    
    [ugh,wgh] = GaussHermite(10) ;
            
    //////////////////////////////////////////////////////////////////////////// 
    ///////// Build precipitation and temperature template at stations /////////
    ////////////////////////////////////////////////////////////////////////////            
       
    TemplateP_sample = mean(PRECIP,'r') ;

    ResiTemplateP_sample = log(TemplateP_sample) - XYZ_precip(:,3)'/ParamPrecip(1);
    P0 = exp(mean(ResiTemplateP_sample));
    ResiTemplateP_sample = ResiTemplateP_sample - mean(ResiTemplateP_sample);  
    
    //////////////////////////////////////////////////////////////////////////// 
    //         Compute residuals to templates at gauges for each day :        //
    // additive for temperature and multiplicative, anamorphosed for rainfall // 
    ////////////////////////////////////////////////////////////////////////////

    // Anamorphosed local scaling factor for precipitation:
    
    LAMBDA = PRECIP./[TemplateP_sample.*.ones(ndays,1)];    
    F0 = sum(LAMBDA==0,'c') / nprecip;

    ixall = find(F0==0);
    ixnone = find(F0==1);
    ixinterm = find ( F0 > 0 & F0 < 1 ) ;    // days with intermittent, non-zero field
    
    LAMBDA(LAMBDA==0) = %nan;    // Skip transformation for zero-values (=> Gibbs sampler)
    F0(ixnone) = %nan;
    MEAN_LAMBDApos = nanmean(LAMBDA,'c');
    
    CV2_LAMBDApos = zeros(ndays,1);
    VAR_Y = zeros(ndays,1);
    MU_Y = zeros(ndays,1);
    
    ALPHA = MeanVariancePrecip(1);
    BETA = MeanVariancePrecip(2);
    GAMMA = MeanVariancePrecip(3);
    DELTA = MeanVariancePrecip(4);
    
    VAR_LAMBDApos = ALPHA .* [ [MEAN_LAMBDApos/BETA].^GAMMA ] .* [ 1 + [MEAN_LAMBDApos/BETA].^(GAMMA-DELTA) ].^(-1) ;
    
//..............................................................................
    VAR_Y = log ( 1 + VAR_LAMBDApos./(MEAN_LAMBDApos.^2) ) ;
    MU_Y = log ( MEAN_LAMBDApos ) - 0.5 * VAR_Y ;
//..............................................................................
            
    ResidualP_sample = %nan*LAMBDA;
    Sy = %nan*F0;

    for r=1:nprecip

        ResidualP_sample ( ixall , r ) = (log(LAMBDA(ixall,r))-MU_Y(ixall))./sqrt(VAR_Y(ixall)) ;
        
    end
    
    // Intermittent days:
        
    LAMBDA(isnan(LAMBDA))=0;
    
    // Structural function for anamorphosed field for all days in weather pattern p :
            
    VARIO_Y = list ( "spherical" , 1 , ParamPrecip(3) , 0 );
    K_ss = CovarianceMatrix ( D_pp , VARIO_Y ) ;
    
    // Computing the nprecip covariance submatrices and their Cholesky factorization
     
    ListCholSubK_ss = list();    
            
    for selected = 1:nprecip
        
        remaining = [1:(selected-1),(selected+1):nprecip];
        ListCholSubK_ss(selected) = KrigCholesky ( K_ss(remaining,remaining) );
        
    end       
    
    // Treating intermittent days
    
    ixinterm = ixinterm(:)';
    
    for j = ixinterm
                    
        // Gibbs sampling :
        
        [Yj,Sy_j] = GibbsAtomLogNorm_fast ( XYZ_precip(:,1:2) , LAMBDA(j,:)' , MU_Y(j) , sqrt(VAR_Y(j)) , K_ss , ListCholSubK_ss );
        
        ResidualP_sample(j,:) = matrix(Yj,1,nprecip);
        Sy(j) = Sy_j;
        
    end
    
    //////////////////////////////////////////////////////////////////////////// 
    ////////////////////////// Compute local forcings //////////////////////////
    ////////////////////////////////////////////////////////////////////////////
        
    // Structural functions for precipitation and temperature templates
    
    VARIO_ResiTemplateP = list ( "spherical" , variance(ResiTemplateP_sample) , ParamPrecip(2) , 0 );
    
    K_pp_template = CovarianceMatrix ( D_pp , VARIO_ResiTemplateP ) ;
    cholK_pp_template = KrigCholesky ( K_pp_template ) ;

    // Structural functions for precipitation and temperature daily residuals

    VARIO_ResidualP = list ( "spherical" , 1 , ParamPrecip(3) , 0 );

    K_pp = CovarianceMatrix ( D_pp , VARIO_ResidualP ) ;
    cholK_pp = KrigCholesky ( K_pp ) ;
    
            X_estim = XYZ_estim(:,1);
            Y_estim = XYZ_estim(:,2);
            
            DX = XYZ_precip(:,1).*.ones(X_estim') - ones(nprecip,1).*.X_estim' ;
            DY = XYZ_precip(:,2).*.ones(Y_estim') - ones(nprecip,1).*.Y_estim' ;
            D_bp = sqrt(DX.^2+DY.^2);                  
                                    
            Elev = XYZ_estim(:,3);
            
        DriftP_estim = P0*exp(Elev/ParamPrecip(1));
        
        K_bp_template = CovarianceMatrix ( D_bp , VARIO_ResiTemplateP ) ;
                    
        [ MU_sk , S2_sk ] = SimpleKriging_fast ( cholK_pp_template , ResiTemplateP_sample' , K_bp_template , VARIO_ResiTemplateP(2) ) ;
        TemplateP_estim = DriftP_estim .* exp( MU_sk + 0.5 * S2_sk );
        
        K_bp = CovarianceMatrix ( D_bp , VARIO_ResidualP ) ;
                    
        //////////////////////////////////////////////////////////////////// 
        /////////////////////////// Loop on days ///////////////////////////
        ////////////////////////////////////////////////////////////////////
        
        for j = 1:ndays                                 
                                            
            if F0(j)<1 then
                                    
                [ MU_sk , S2_sk ] = SimpleKriging_fast ( cholK_pp , ResidualP_sample(j,:)' , K_bp , 1.0 ) ;
                                    
                if F0(j)>0 then
                       
                    Ugh = MU_sk(:) .*. ones(ugh) + sqrt(S2_sk(:)) .*. ugh ;    // matrix [npts,nGH]
                    Wgh = ones(MU_sk(:)) .*. wgh ;                             // matrix [npts,nGH]
                      
                    // Compute Lambda(Ugh) = ( Fz^(-1) o N ) (U)
                    
                    Lambda_gh = Anamorph_LN ( Ugh , MU_Y(j)*ones(Ugh) , sqrt(VAR_Y(j))*ones(Ugh) , F0(j)*ones(Ugh) );
                    
                    // Gauss-Hermite quadrature
                    
                    Lambda_estim = (1/sqrt(%pi))*sum ( Wgh.*Lambda_gh , 'c' );   
                    Lambda_estim = matrix(Lambda_estim,size(MU_sk));                    
               
                else
               
                    Lambda_estim = exp( MU_Y(j) + sqrt(VAR_Y(j))* MU_sk + VAR_Y(j)*S2_sk/4 );
               
                end
                
            else
                
                Lambda_estim = zeros(TemplateP_estim);
                
            end

            P_estim = TemplateP_estim.*Lambda_estim;
            LocalP(j,:) = matrix(P_estim,1,nestim);
                        
        end        //    end loop on days
    
endfunction
