// 
// Copyright (C) 2013 UMPC
// 
// This file is part of CLaRiNet.
//  
// CLaRiNet is free software: you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// CLaRiNet is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with CLaRiNet.  If not, see <http://www.gnu.org/licenses/>.
function LocalP = BuildLocalRainfall2 ( PRECIP , XYZ_precip , ScaleHeight , MeanVariancePrecip , VARIO_ResiTemplateP , SlopeVario , F0 , MU_Y , VAR_Y , ResidualP_sample , XYZ_estim )
    
    // PRECIP            :     Matrix [ndays,nprecip]    :    Daily rainfall time series at gauges
    // XYZ_precip        :     Matrix [nprecip,3]        :    X,Y,Z coordinates of rain gauges
    // XYZ_estim         :     Matrix [nestim,3]         :    XYZ of ungaged locations
    //
    // ScaleHeight       :     Integer                   
    // MeanVariancePrecip :    Vector [4]                :    Parameters of the mean-variance relationship of the local scaling factor
    
    PRECIP(PRECIP<0) = %nan ;
    
    if or(isnan(PRECIP)) then
        error("Les series pluviometriques ne doivent pas comporter de lacunes");
    end
    
    [ndays,nprecip] = size(PRECIP);
    nestim = size(XYZ_estim,'r');
    
    if or( size(XYZ_precip) ~= [nprecip,3] ) then
        error("Les arguments 1 et 3 ont des dimensions incompatibles");
    end

    // Dimensioning output:
    
    LocalP = zeros(ndays,nestim);

    //////////////////////////////////////////////////////////////////////////// 
    ///////////// Compute sample-sample symmetric distance matrices ////////////
    ////////////////////////////////////////////////////////////////////////////
    
    DX = XYZ_precip(:,1)'.*.ones(nprecip,1)-XYZ_precip(:,1).*.ones(1,nprecip);
    DY = XYZ_precip(:,2)'.*.ones(nprecip,1)-XYZ_precip(:,2).*.ones(1,nprecip);
    D_pp = sqrt(DX.^2+DY.^2);           
    
    //////////////////////////////////////////////////////////////////////////// 
    /// Computing Gauss-Hermite quadrature abcissas and weights once for all ///
    ////////////////////////////////////////////////////////////////////////////
    
    [ugh,wgh] = GaussHermite(10) ;
    
    //////////////////////////////////////////////////////////////////////////// 
    /////////// Structural functions and associated kriging matrices ///////////
    ////////////////////////////////////////////////////////////////////////////            
      
    K_pp_template = CovarianceMatrix ( D_pp , VARIO_ResiTemplateP ) ;
    cholK_pp_template = KrigCholesky ( K_pp_template ) ;    
           
    Gamm_pp = SlopeVario * D_pp ;
    FactVar = max(Gamm_pp);
    Gamm_pp = Gamm_pp/FactVar;
    MatKrig_pp = [-Gamm_pp,ones(nprecip,1);ones(1,nprecip),0];
    [LU_pp,IPIV_pp] = KrigBunchKaufman ( MatKrig_pp ) ;
            
    //////////////////////////////////////////////////////////////////////////// 
    ///////// Build precipitation and temperature template at stations /////////
    ////////////////////////////////////////////////////////////////////////////            
       
    TemplateP_sample = mean(PRECIP,'r') ;

    ResiTemplateP_sample = log(TemplateP_sample) - XYZ_precip(:,3)'/ScaleHeight;
    P0 = exp(mean(ResiTemplateP_sample));
    ResiTemplateP_sample = ResiTemplateP_sample - mean(ResiTemplateP_sample);  
     
    //////////////////////////////////////////////////////////////////////////// 
    ////////////////////////// Compute local forcings //////////////////////////
    ////////////////////////////////////////////////////////////////////////////
      
    X_estim = XYZ_estim(:,1);
    Y_estim = XYZ_estim(:,2);
    
    DX = XYZ_precip(:,1).*.ones(X_estim') - ones(nprecip,1).*.X_estim' ;
    DY = XYZ_precip(:,2).*.ones(Y_estim') - ones(nprecip,1).*.Y_estim' ;
    D_pe = sqrt(DX.^2+DY.^2);                  
                            
    Elev = XYZ_estim(:,3);
        
    DriftP_estim = P0*exp(Elev/ScaleHeight);
    
    K_pe_template = CovarianceMatrix ( D_pe , VARIO_ResiTemplateP ) ;
    
    Gamm_pe = SlopeVario * D_pe ;
    Gamm_pe = Gamm_pe/FactVar;
    
    [ W_sk , S2_sk ] = SimpleKriging_fast ( cholK_pp_template , K_pe_template , VARIO_ResiTemplateP(2) ) ;
    MU_sk = ResiTemplateP_sample * W_sk ; 
    TemplateP_estim = DriftP_estim .* exp( MU_sk + 0.5 * S2_sk );
                
    //////////////////////////////////////////////////////////////////// 
    /////////////////////////// Loop on days ///////////////////////////
    ////////////////////////////////////////////////////////////////////
    
    [ W_ok , S2_ok ] = OrdinaryKriging_fast ( LU_pp , IPIV_pp , Gamm_pe , FactVar ) ;
    
    for j = 1:ndays                                 
                                        
        if F0(j)<1 then
                                
            MU_ok = ResidualP_sample(j,:) * W_ok(1:nprecip,:) ;
                                
            if F0(j)>0 then
                   
                Ugh = MU_ok(:) .*. ones(ugh) + sqrt( 2.0 * S2_ok(:)) .*. ugh ;    // matrix [npts,nGH]
                Wgh = ones(MU_ok(:)) .*. wgh ;                             // matrix [npts,nGH]
                  
                // Compute Lambda(Ugh) = ( Fz^(-1) o N ) (U)
                
                Lambda_gh = Anamorph_LN ( Ugh , MU_Y(j)*ones(Ugh) , sqrt(VAR_Y(j))*ones(Ugh) , F0(j)*ones(Ugh) );
                
                // Gauss-Hermite quadrature
                
                Lambda_estim = (1/sqrt(%pi))*sum ( Wgh.*Lambda_gh , 'c' );   
                Lambda_estim = matrix(Lambda_estim,size(MU_ok));                    
           
            else
           
                Lambda_estim = exp( MU_Y(j) + sqrt(VAR_Y(j))* MU_ok + VAR_Y(j)*S2_ok/4 );
           
            end
            
        else
            
            Lambda_estim = zeros(TemplateP_estim);
            
        end

        P_estim = TemplateP_estim.*Lambda_estim;
        LocalP(j,:) = matrix(P_estim,1,nestim);
                    
    end        //    end loop on days
    
endfunction
