// 
// Copyright (C) 2013 UMPC
// 
// This file is part of CLaRiNet.
//  
// CLaRiNet is free software: you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// CLaRiNet is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with CLaRiNet.  If not, see <http://www.gnu.org/licenses/>.
function LocalT = BuildLocalTemperature ( TEMPER , XYZ_temper , XYZ_estim , ParamTemper )
    
    // TEMPER            :     Matrix [ndays,ntemper]    :    Daily temperature time series at stations
    // XYZ_precip        :     Matrix [ntemper,3]        :    X,Y,Z coordinates of temperature stations
    // XYZ_estim         :     Matrix [nestim,3]         :    XYZ of ungaged locations
    //
    // ParamTemper       :     Vector [3]                :    ParamTemper(1)    =>    Temperature lapse rate
    //                                                        ParamTemper(2)    =>    variogram range for temperature template residuals
    //                                                        ParamTemper(3)    =>    variogram range for daily offset
        
    if or(isnan(TEMPER)) then
        error("Les series de temperature ne doivent pas comporter de lacunes");
    end
    
    [ndays,ntemper] = size(TEMPER);
    nestim = size(XYZ_estim,'r');
         
    if or( size(XYZ_temper) ~= [ntemper,3] ) then
        error("Les arguments 1 et 3 ont des dimensions incompatibles");
    end
        
    //////////////////////////////////////////////////////////////////////////// 
    ///////////// Compute sample-sample symmetric distance matrices ////////////
    ////////////////////////////////////////////////////////////////////////////
        
    DX = XYZ_temper(:,1)'.*.ones(ntemper,1)-XYZ_temper(:,1).*.ones(1,ntemper);
    DY = XYZ_temper(:,2)'.*.ones(ntemper,1)-XYZ_temper(:,2).*.ones(1,ntemper);
    D_tt = sqrt(DX.^2+DY.^2);
        
    // Dimensioning output:
    
    LocalT = zeros(ndays,nestim);
    
    //////////////////////////////////////////////////////////////////////////// 
    ///////// Build precipitation and temperature template at stations /////////
    ////////////////////////////////////////////////////////////////////////////
                
    TemplateT_sample = mean(TEMPER,'r') ;

    ResiTemplateT_sample = TemplateT_sample - XYZ_temper(:,3)'* ParamTemper(1);
    T0 = mean(ResiTemplateT_sample);  
    ResiTemplateT_sample = ResiTemplateT_sample - mean(ResiTemplateT_sample);
    
    //////////////////////////////////////////////////////////////////////////// 
    //         Compute residuals to templates at gauges for each day :        //
    // additive for temperature and multiplicative, anamorphosed for rainfall // 
    ////////////////////////////////////////////////////////////////////////////
    
    // Temperature:
    
    ResidualT_sample = TEMPER - TemplateT_sample.*.ones(ndays,1);
    MResidualT_sample = mean(ResidualT_sample,'c');
    CResidualT_sample = ResidualT_sample - MResidualT_sample.*.ones(1,ntemper);

    //////////////////////////////////////////////////////////////////////////// 
    ////////////////////////// Compute local forcings //////////////////////////
    ////////////////////////////////////////////////////////////////////////////
                
    // Structural functions for precipitation and temperature templates
    
    VARIO_ResiTemplateT = list ( "spherical" , variance(ResiTemplateT_sample) , ParamTemper(2) , 0 );

    K_tt_template = CovarianceMatrix ( D_tt , VARIO_ResiTemplateT ) ;
    cholK_tt_template = KrigCholesky ( K_tt_template ) ;

    // Structural functions for precipitation and temperature daily residuals

    VARIO_ResidualT = list ( "spherical" , 1 , ParamTemper(3) , 0 );

    K_tt = CovarianceMatrix ( D_tt , VARIO_ResidualT ) ;
    cholK_tt = KrigCholesky ( K_tt ) ;
   
        X_estim = XYZ_estim(:,1);
        Y_estim = XYZ_estim(:,2);
                
        DX = XYZ_temper(:,1).*.ones(X_estim') - ones(ntemper,1).*.X_estim' ;
        DY = XYZ_temper(:,2).*.ones(Y_estim') - ones(ntemper,1).*.Y_estim' ;
        D_bt = sqrt(DX.^2+DY.^2);
        
        Elev = XYZ_estim(:,3);
        
        DriftT_estim = T0 + Elev*ParamTemper(1);
        
        K_bt_template = CovarianceMatrix ( D_bt , VARIO_ResiTemplateT ) ;
        
        [ MU_sk , S2_sk ] = SimpleKriging_fast ( cholK_tt_template , ResiTemplateT_sample' , K_bt_template , VARIO_ResiTemplateT(2) ) ;
        TemplateT_estim = DriftT_estim + MU_sk ;
        
        K_bt = CovarianceMatrix ( D_bt , VARIO_ResidualT ) ;
                    
        //////////////////////////////////////////////////////////////////// 
        /////////////////////////// Loop on days ///////////////////////////
        ////////////////////////////////////////////////////////////////////
        
        for j = 1:ndays                                 

            [ MU_sk , S2_sk ] = SimpleKriging_fast ( cholK_tt , CResidualT_sample(j,:)' , K_bt , variance(CResidualT_sample(j,:)) ) ;
            
            T_estim = TemplateT_estim + MU_sk + MResidualT_sample(j) ;
                                            
            LocalT(j,:) = matrix(T_estim,1,nestim);
                        
        end        //    end loop on days
                    
    
endfunction
