// 
// Copyright (C) 2013 UMPC
// 
// This file is part of CLaRiNet.
//  
// CLaRiNet is free software: you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// CLaRiNet is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with CLaRiNet.  If not, see <http://www.gnu.org/licenses/>.
function [ SlopeVario , F0 , MU_Y , VAR_Y , ResidualP_sample ] = DailyP_variography ( PRECIP , XYZ_precip , MeanVarianceLambda )
        
    PRECIP(PRECIP<0) = %nan ;
    
    if or(isnan(PRECIP)) then
        error("Les series pluviometriques ne doivent pas comporter de lacunes");
    end
    
    [ndays,nprecip] = size(PRECIP);
    
    if or( size(XYZ_precip) ~= [nprecip,3] ) then
        error("Les arguments 1 et 3 ont des dimensions incompatibles");
    end

    //////////////////////////////////////////////////////////////////////////// 
    ///////////// Compute sample-sample symmetric distance matrices ////////////
    ////////////////////////////////////////////////////////////////////////////
    
    DX = XYZ_precip(:,1)'.*.ones(nprecip,1)-XYZ_precip(:,1).*.ones(1,nprecip);
    DY = XYZ_precip(:,2)'.*.ones(nprecip,1)-XYZ_precip(:,2).*.ones(1,nprecip);
    D_pp = sqrt(DX.^2+DY.^2);
                        
    //////////////////////////////////////////////////////////////////////////// 
    ///////// Build precipitation and temperature template at stations /////////
    ////////////////////////////////////////////////////////////////////////////            
       
    TemplateP_sample = mean(PRECIP,'r') ;

    //////////////////////////////////////////////////////////////////////////// 
    //         Compute residuals to templates at gauges for each day :        //
    // additive for temperature and multiplicative, anamorphosed for rainfall // 
    ////////////////////////////////////////////////////////////////////////////

    // Anamorphosed local scaling factor for precipitation:
    
    LAMBDA = PRECIP./[TemplateP_sample.*.ones(ndays,1)];    
    F0 = sum(LAMBDA==0,'c') / nprecip;

    ixall = find(F0==0);
    ixnone = find(F0==1);
    ixinterm = find ( F0 > 0 & F0 < 1 ) ;    // days with intermittent, non-zero field
    
    LAMBDA(LAMBDA==0) = %nan;    // Skip transformation for zero-values (=> Gibbs sampler)
    F0(ixnone) = %nan;
    MEAN_LAMBDApos = nanmean(LAMBDA,'c');
    
    CV2_LAMBDApos = zeros(ndays,1);
    VAR_Y = zeros(ndays,1);
    MU_Y = zeros(ndays,1);
    
    ALPHA = MeanVarianceLambda(1);
    BETA = MeanVarianceLambda(2);
    GAMMA = MeanVarianceLambda(3);
    DELTA = MeanVarianceLambda(4);
    
    VAR_LAMBDApos = ALPHA .* [ [MEAN_LAMBDApos/BETA].^GAMMA ] .* [ 1 + [MEAN_LAMBDApos/BETA].^(GAMMA-DELTA) ].^(-1) ;
    
//..............................................................................
    VAR_Y = log ( 1 + VAR_LAMBDApos./(MEAN_LAMBDApos.^2) ) ;
    MU_Y = log ( MEAN_LAMBDApos ) - 0.5 * VAR_Y ;
//..............................................................................
            
    ResidualP_sample = %nan*LAMBDA;
    Sy = %nan*F0;

    for r=1:nprecip

        ResidualP_sample ( ixall , r ) = (log(LAMBDA(ixall,r))-MU_Y(ixall))./sqrt(VAR_Y(ixall)) ;
        
    end
    
    //////////////////////////////////////////////////////////////////////////// 
    //////// Variographie des résidus à l'ébauche (intermède musical 2) ////////
    ////////////////////////////////////////////////////////////////////////////            

    h = zeros(nprecip*(nprecip-1)/2,1);
    g = zeros(h);    
    i = 0;
            
    for p1=1:(nprecip-1)       
        for p2=(p1+1):nprecip
            i=i+1;
            h(i) = D_pp(p1,p2);
            accr = ResidualP_sample(ixall,p1) - ResidualP_sample(ixall,p2) ;
            g(i) = 0.5*nanmean(accr.^2);
        end       
    end
   
    SlopeVario = h\g ;
    
    Gamm_pp = SlopeVario * D_pp ;
    FactVar = max(Gamm_pp);
    Gamm_pp = Gamm_pp/FactVar;
    MatKrig_pp = [-Gamm_pp,ones(nprecip,1);ones(1,nprecip),0];
//    [LU_pp,IPIV_pp] = KrigBunchKaufman ( MatKrig_pp ) ;
    
    // Intermittent days:
        
    LAMBDA(isnan(LAMBDA))=0;
        
    // Computing the nprecip "jackknifed" kriging weights for the Gibbs sampler
    
    List_JK_weights = list();
    List_JK_varkrig = list();
                 
    for selected = 1:nprecip
        
        remaining = [1:(selected-1),(selected+1):nprecip];
        MK = [-Gamm_pp(remaining,remaining),ones(nprecip-1,1);ones(1,nprecip-1),0];
        [LU,IPIV] = KrigBunchKaufman ( MK );
	gsr = Gamm_pp(remaining,selected);
        B = [-gsr;1] ;                                  // Right-hand side

	lambda = KrigSolveOK(LU,IPIV,B) ;

        List_JK_weights(selected) = lambda ; 
        List_JK_varkrig(selected) = FactVar * [ gsr' * lambda(1:($-1)) - lambda($) ] ;
        
    end       
    
    // Treating intermittent days
    
    ixinterm = ixinterm(:)';
    
    for j = ixinterm
                    
        // Gibbs sampling :
        
        [Yj,Sy_j] = GibbsAtomLogNorm_intr ( XYZ_precip(:,1:2) , LAMBDA(j,:)' , MU_Y(j) , sqrt(VAR_Y(j)) , List_JK_weights , List_JK_varkrig );
        
        ResidualP_sample(j,:) = matrix(Yj,1,nprecip);
        Sy(j) = Sy_j;
        
    end 
    
    F0(ixnone)=1;
      
    
endfunction
