// 
// Copyright (C) 2013 UMPC
// 
// This file is part of CLaRiNet.
//  
// CLaRiNet is free software: you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// CLaRiNet is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with CLaRiNet.  If not, see <http://www.gnu.org/licenses/>.
function [Y,Sy] = GibbsAtomLogNorm_intr ( XY , NGValues , m , s , List_JK_weights , List_JK_varkrig )

    ////////////////////////////////////////////////////////////////////////////
    // INPUT VARIABLES :                                                      //
    //                                                                        //
    //    XY :         [N,2] matrix :    coordinates of conditioning points   //
    //    NGValues :   [N,1] vector :    values of the non-gaussian field at  //
    //                                   conditioning locations. CDF of this  //
    //                                   field is a log-normal distribution   //
    //                                   with atomic component at zero.       //
    //                                                                        //
    // OUTPUT VARIABLES :                                                     //
    //                                                                        //
    //    Y :         [N,1] vector :     Anamorphosed values                  //
    //    m :         scalar       :     logaritmic mean (location parameter) // 
    //    s :         scalar       :     logaritmic std.dev (scale parameter) //
    //    F0 :        scalar       :     Empirical frequency of zero-values   //
    ////////////////////////////////////////////////////////////////////////////
    
    N = length(NGValues);                    // Number of conditioning points
    z = (NGValues==0);                       // TRUE where the sampled field has its atomic value
    indz = find(z);                          // indices of zero values in vector NGValues 
    Nz = length(indz);                       // number of zero values
    
//    m = mean(log(NGValues(~z)));             // logaritmic mean of non-zero values
//    s = stdev(log(NGValues(~z)));            // logaritmic st.dev of non-zero values
    
    if Nz>0 then
//        F0 = ( Nz - 3/8 ) / ( N + 1 - 2*3/8 );
        F0 = Nz/N;
        Sy = cdfnor("X",0,1,F0,1-F0);        // Sy = N^-1 ( F0 )
    else
        F0 = 0;
        Sy = -%inf;       
    end
    
    // Applying the inverse of the anamorphosis function :
    
    f = zeros(NGValues) ;
    f(~z) = F0+0.5*(1-F0)*erfc(-(log(NGValues(~z))-m)/(s*sqrt(2))) ;    // non-zero points
    f(z) = F0 * rand(NGValues(z));                                      // zero points

    // Y = sqrt(2)*erfinv(2*f-1)
    
    Y = zeros(f);
    
    argerfinv = 2*f-1;
    ixinf = (1-argerfinv)<1e-15;
    
    Y(~ixinf) = sqrt(2)*erfinv(2*f(~ixinf)-1);
    
    ln1_x2 = log(2) + log(1-F0) - 0.5*log(%pi) - [(log(NGValues(ixinf))-m).^2]/(2*s*s) ;
    approx = sqrt( sqrt ( ( 2/(%pi*0.147) + 0.5*ln1_x2 ).^2 - ln1_x2/0.147  ) - 2/(%pi*0.147) - 0.5*ln1_x2 );
    
    Y(ixinf) = sqrt(2)*approx;
           
    if Nz == 0        // if no zero-values, just leave!
        Sy = -%inf ;
        return
    end
    
    //////////////////////////////////////////////////////////////////////////// 
    ////////////////////////////// Gibbs sampling //////////////////////////////
    //////////////////////////////////////////////////////////////////////////// 

    iterate = %T ;
    eps = 1e-6;
    Y0 = Y ;
                
    while iterate 

        [sh,shuffle] = gsort( rand(1,Nz) , 'g' , 'i' ) ;
        permut_indz = matrix( indz(shuffle) , 1 , Nz ) ;     // permutation of zero-indices

        for selected = permut_indz 
                   
            remaining = [1:(selected-1),(selected+1):N] ;   // remaining points
                
            lambda = List_JK_weights(selected) ;            // Pre-computed solution of ordinary kriging
            v_OK = List_JK_varkrig(selected) ;     	    // Pre-computed OK variance
                
            m_OK = Y(remaining)' * lambda(1:($-1)) ;        // OK estimation at the selected point
            
            arg = (Sy-m_OK)/sqrt(2*v_OK) ;
            
            if arg>-5 then
                Y(selected) = m_OK - sqrt(2*v_OK/%pi) * exp(-arg^2) / (1+erf(arg)) ;    // Expected value E ( y | y < Sy )
            else
                Y(selected) = m_OK + sqrt(2*v_OK) * arg ;                                 // Asymptotic expression in case where Sy << m_SK 
            end
        
        end
    
        iterate = norm ( Y(z) - Y0(z) ) / norm( Y0(z) ) <= (eps*eps) ;
        Y0 = Y ;
                
   end
       
endfunction
