// 
// Copyright (C) 2013 UMPC
// 
// This file is part of CLaRiNet.
//  
// CLaRiNet is free software: you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// CLaRiNet is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with CLaRiNet.  If not, see <http://www.gnu.org/licenses/>.
function [S,N] = MeasureCollapse (X,SETS,D,withplot)
    
    // SETS : objet de type liste dont les éléments sont des matrices à 3 colonnes [logai,logpi,dyi]
    // D : vecteur de taille k=length(SETS) donnant la dimension du système pour chaque élément de SETS
    //
    // X = [ bet , gam ] : paramètres d'échelle
    // F(a) = Pr(A<=a) = 1 - (a^-bet)*f(a*D^gam)        pour a>a0
    // On note     
    //                xij = log ( aij * Di^-gam ) = logaij - gam*log(Di)
    //                yij = log ( (aij^bet) * ( 1 - F(aij) ) ) = bet * logaij + logpij
    //
    // Si la relation d'échelle est valable, yij = (log o f o exp) (xij) => superposition de tous les jeux i=1..k
    //
    // On calcule une statistique S indiquant la qualité de la superposition ("collapse") pour le jeu de paramètres X testé
    // REFERENCE : 
    // Houdayer, J., and A.K. Hartmann (2004) Low-temperature behavior of two-dimensional Gaussian Ising spin glasses, Phys. Rev. B 70, 014418
    // http://dx.doi.org/10.1103/PhysRevB.70.014418
    
    
    
    k = length(SETS);
    bet = X(1);
    gam = X(2);
               
    // Application de la transformation d'échelle sur chaque jeu de données
    
    TSETS = list();
        
    if withplot then
 //       clf();
        Col = jetcolormap(k);
        Col = flipdim(Col,1);
    end
        
        
    for i=1:k       
            
        logai = SETS(i)(:,1);
        logpi = SETS(i)(:,2);
        dyi = SETS(i)(:,3);       
         
        xi = logai - gam*log(D(i)) ;
        yi = bet*logai + logpi ;
        
        TSETS(i) = [xi,yi,dyi];
        
        if withplot then

            plot(xi,yi,'.','color',Col(i,:));
            f = gcf();
            f.children(1).children(1).children(1).line_mode = "off";
            f.children(1).children(1).children(1).mark_size = 4;
//            plot(xi,[yi+dyi,yi-dyi],'-','color',Col(i,:));
                    
        end
        
    end    

    // Calcul de la statistique S 
    
    S = 0;
    N = 0;
    
    for i=1:k       
        
        for j=1:size(TSETS(i),'r')    // boucle sur les points xij
            
            xij = TSETS(i)(j,1);
            yij = TSETS(i)(j,2);
            dyij = TSETS(i)(j,3);
           
            // Collecte des points autour de xij
           
            L = [];
           
            for ii=1:k
                
                if ii==i then
                    break;
                end
                
                d = diff( sign( TSETS(ii)(:,1)-xij ) );
                jj = find(d,1);
            
                if  ~isempty(jj) then                             
                    L = [ L ; TSETS(ii)(jj+[0 1],:)] ;
                end
            
            end
        
            // Régression sur les points de L
            
            if ~isempty(L) then
            
                xl = L(:,1);
                yl = L(:,2);
                wl = L(:,3).^(-2);
                
                K = sum(wl);
                Kx = sum(wl.*xl);
                Ky = sum(wl.*yl);
                Kxx = sum(wl.*xl.*xl);
                Kxy = sum(wl.*xl.*yl);
                Delta = K*Kxx - Kx*Kx;
                
                Yij = (Kxx*Ky - Kx*Kxy)/Delta + xij * (K*Kxy - Kx*Ky)/Delta ;
                dYij2 = (Kxx - 2*xij*Kx + K*xij^2 )/Delta ;
                
                S = S + [(yij-Yij)^2] / [dyij^2 + dYij2 ];
                N = N + 1;
            
            end            
        
        end
           
    end
    
    S = sqrt(S/N);
    
    if (withplot) then
        xtitle(sprintf("S = %.3f",S),"","");
    end
    
endfunction
