// 
// Copyright (C) 2013 UMPC
// 
// This file is part of CLaRiNet.
//  
// CLaRiNet is free software: you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// CLaRiNet is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with CLaRiNet.  If not, see <http://www.gnu.org/licenses/>.
function [ BETA , R , VARIO ] = RK_fitdeterm ( XY_sample , Z_sample , AUX_sample , typevario , withnugget , nbpoints , moving , nit )
    
    ////////////////////////////////////////////////////////////////////////////
    //////// IDENTIFICATION D'UN MODELE LINEAIRE MULTI-VARIE A RESIDUS /////////
    ////////////////////////////////////////////////////////////////////////////
        
    // XY_sample   :  matrice [n,2] des coordonnées des points échantillonnés
    // Z_sample    :  vecteur [n,1] des valeurs de la variable principale aux points échantillonnés
    // AUX_sample  :  matrice [n,p] des p variables auxiliaires aux n points échantillonnés
    // typevario   :  "spherical" , "exponential" ou "gaussian" 
    // withnugget  :  booléen. %T si optimisation d'un effet pépite, %F sinon
    // nbpoints    :  nombre de points pour définir un intervalle de distance dans le variogramme empirique
    // moving      :  booléen. %T pour définir des intervalles glissants, %F pour définir des intervalles disjoints
    // nit         :  nombre d'itérations Variographie->GLS
    
    // BETA        :  vecteur [p+1,1] des coefficients de régression
    // R           :  vecteur [n,1] des résidus aux points échantillonnés
    // VARIO       :  variogramme estimé. list(type,Sill,Range,Nugget)
    
    typevario = convstr ( part(typevario,1:1) , 'l' );
    
    ////////////////////////////////////////////////////////////////////////////
    ////////////// Centrage et réduction de toutes les grandeurs ///////////////
    //////////////  avec leurs moments aux points échantillonnés ///////////////
    ////////////////////////////////////////////////////////////////////////////
    
    [n,p] = size(AUX_sample) ;

    m_Z = mean(Z_sample) ;
    s_Z = stdev(Z_sample) ;
    
    m_AUX = mean(AUX_sample,'r') ;
    M_AUX = m_AUX.*.ones(n,1);
    s_AUX = stdev(AUX_sample,'r') ;
    S_AUX = s_AUX.*.ones(n,1);
    
    A = [ [ AUX_sample - M_AUX ] ./ S_AUX , ones(n,1) ] ;
    
    XY = XY_sample - mean(XY_sample,'r').*.ones(n,1) ;
    
    ////////////////////////////////////////////////////////////////////////////
    ////////// Première estimation OLS des coefficients de régression //////////
    ////////////////////////////////////////////////////////////////////////////
            
    ZZ = ( Z_sample - m_Z ) / s_Z ;    
    BETA = A \ ZZ ;
    R = A*BETA - ZZ ;
    
    ////////////////////////////////////////////////////////////////////////////
    ///////////////////////// Estimation GLS itérative /////////////////////////
    ////////////////////////////////////////////////////////////////////////////      
    
    for it = 1:nit
           
        // Variographie ... (intermède musical) --------------------------------
       
        [ h_window , g , D ] = VarioEmpirique ( XY , R , nbpoints , moving ) ;
        h = mean ( h_window , 'c' ) ;
        
        Sill = variance(R) ;
                        
        if withnugget then
            
            X0 = [ 0 0 ] ;
            [fopt,Xopt,gropt] = leastsq ( 2 , list ( VarioDiff_w_nugget , Sill , typevario , h , g ) , X0 , 'ar' , 100 , 100 );
            Range = max(h) * 0.5 * ( 1 + tanh ( Xopt(1) ) );
            Nugget = Sill * 0.5 * ( 1 + tanh ( Xopt(2) ) );
            
        else
            
           Nugget = 0 ;
           X0 = [ 0 ] ;
           [fopt,Xopt,gropt] = leastsq ( 2 , list ( VarioDiff_wo_nugget , Nugget , Sill , typevario , h , g ) , X0 , 'ar' , 100 , 100 );        
           Range = max(h) * 0.5 * ( 1 + tanh ( Xopt(1) ) );
           
        end
       
        // Estimation GLS ------------------------------------------------------
       
        gr0 = Nugget/Sill ;
          
        select typevario
        case 'e' then    // variogramme exponentiel
            gr = gr0 + (1-gr0) * [ 1-exp(-D/Range) ];
        case 's' then    // variogramme sphérique
            gr = ones(D);
            gr(D<Range) = gr0 + (1-gr0) * [ 1.5*D(D<Range)/Range-0.5*(D(D<Range)/Range).^3 ] ;
        case 'e' then    // variogramme gaussien
            gr = gr0 + (1-gr0) * [ 1-exp(-[D/Range].^2) ];
        end
    
        gr = Sill*(1-gr) ; //    Variogramme -> covariance
        
        B = gr^(-1) ;
        C = [A'*B*A]^(-1) ;
        
        BETA = C*A'*B*ZZ ;  
        R = A*BETA - ZZ ;
    
    end
   
    ////////////////////////////////////////////////////////////////////////////
    ////////////// Dé-normalisation du vecteur BETA et des résidus /////////////
    ////////////////////////////////////////////////////////////////////////////
   
    BETA(1:p) = BETA(1:p) ./ s_AUX' ;
    BETA(p+1) = BETA(p+1) - sum ( BETA(1:p) .* m_AUX' );
    BETA = BETA * s_Z ;
    BETA(p+1) = BETA(p+1) + m_Z ;
    
    R = R * s_Z ;
    Sill = Sill * (s_Z^2) ;
    VARIO = list(typevario,Sill,Range,Nugget);
    
endfunction
